"use strict";

const argx = require('argx'),
    apemanfile = require('apemanfile'),
    apemancwd = require('apemancwd'),
    arrayreduce = require('arrayreduce'),
    arrayfilter = require('arrayfilter'),
    apemandocker = require('apemandocker'),
    async = require('async'),
    runDply = require('./deploying/run_dply'),
    listDply = require('./deploying/list_dply'),
    ttyDply = require('./deploying/tty_dply'),
    logDply = require('./deploying/log_dply'),
    removeDply = require('./deploying/remove_dply'),
    execCmd = require('./deploying/exec_cmd');

let concatArray = arrayreduce.arrayConcat(),
    rejectEmpty = arrayfilter.emptyReject();

/** @lends apemanDply */
function apemanDply(names, options, callback) {
    let args = argx(arguments);
    callback = args.pop('function') || argx.noop;
    options = args.pop('object') || {};
    names = args.remain().filter(rejectEmpty).reduce(concatArray, []);

    let docker = apemandocker.create(),
        cwd = apemancwd.create({prefix: 'apeman-dply'});

    async.waterfall([
        (callback) => {
            apemanfile.load(options.configuration, callback);
        },
        (apemanfile, callback) => {
            cwd.chdir(apemanfile.$cwd);

            let duplicate = [
                    options.list,
                    options.delete,
                    options.rerun,
                    options.tty,
                    options.exec,
                    options.print,
                    options.printon
                ].filter((command) => {
                    return !!command;
                }).length > 1;
            if (duplicate) {
                callback(new Error(`Invalid option: ${JSON.stringify(options, null, 2)}`));
                return;
            }
            if (options.rerun) {
                async.series([
                    (callback) => {
                        removeDply(apemanfile, names, {
                            force: options.force
                        }, callback);
                    },
                    (callback) => {
                        runDply(apemanfile, names, {
                            force: options.force
                        }, callback);
                    }
                ], callback);
                return;
            }

            if (options.exec) {
                execCmd(apemanfile, names, options.exec, {}, callback);
                return;
            }

            if (options.list) {
                listDply(apemanfile, names, {}, callback);
                return;
            }
            if (options.delete) {
                removeDply(apemanfile, names, {
                    force: options.force
                }, callback);
                return;
            }

            if (options.tty) {
                ttyDply(apemanfile, names, {}, callback);
                return;
            }

            if (options.print) {
                logDply(apemanfile, names, {}, callback);
                return;
            }

            if (options.printon) {
                logDply(apemanfile, names, {
                    follow: true
                }, callback);
                return;
            }

            runDply(apemanfile, names, {
                force: options.force
            }, callback);
        }
    ], (err) => {
        cwd.restoreAll();
        callback(err);
    });
}

module.exports = apemanDply;
