/**
 * @memberof module:apeman-dply
 * @function getDplys
 * @param {Apemanfile} apemanfile
 * @param {string[]} names
 * @param {function} callback
 */

"use strict";

const arrayreduce = require('arrayreduce'),
    arrayfilter = require('arrayfilter'),
    stringcase = require('stringcase'),
    minimatch = require('minimatch'),
    apemandocker = require('apemandocker'),
    Dply = require('./dply'),
    nameDply = require('./name_dply'),
    findout = require('findout');

let rejectEmpty = arrayfilter.emptyReject(),
    concatArray = arrayreduce.arrayConcat();

/** @lends getDplys */
function getDplys(apemanfile, names) {
    let $pkg = apemanfile.$pkg;
    if (!$pkg) {
        throw new Error(`$pkg not found.`);
    }

    let $dplys = apemanfile.$dplys || {};

    return Object.keys($dplys)
        .filter((filtering) => {
            let hasFilter = names && names.length > 0;
            if (!hasFilter) {
                return true;
            }
            for (let i = 0, len = names.length; i < len; i++) {
                let hit = minimatch(filtering, names[i]);
                if (hit) {
                    return true;
                }
            }
            return false;
        })
        .map((name) => {
            let dply = $dplys[name];
            return [].concat(dply || [])
                .reduce(concatArray, [])
                .filter(rejectEmpty)
                .map((conf, i) => {
                    if (typeof(conf) === 'string') {
                        return findout(conf);
                    }
                    return conf;
                })
                .map((conf, i) => {
                    if (conf.infra) {
                        console.warn('conf.infra is deprecated. use conf.infr instead.');
                        conf.infr = conf.infra;
                    }
                    return new Dply({
                        name: name,
                        hostname: conf.hostname,
                        container: nameDply(name, $pkg),
                        port: conf.port,
                        mount: conf.mount,
                        env: conf.env,
                        index: [i].join('-'),
                        image: apemandocker.nameImage($pkg, conf.infr)
                    });
                })
        })
        .sort((a, b) => {
            return String(a.name).localeCompare(String(b.name));
        })
        .reduce(concatArray, []);
}

module.exports = getDplys;