/**
 * @function removeDply
 */
"use strict";

const argx = require('argx'),
    path = require('path'),
    async = require('async'),
    apemandocker = require('apemandocker'),
    getDplys = require('./get_dplys'),
    RemoveLogger = require('../logging/remove_logger');

/** @lends removeDply */
function removeDply(apemanfile, names, options, callback) {
    let args = argx(arguments);
    callback = args.pop('function') || argx.noop;
    options = args.pop('object') || {};

    let docker = apemandocker.create({}),
        logger = new RemoveLogger({}),
        dplys = getDplys(apemanfile, names);

    async.series([
        (callback) => {
            async.eachSeries([].concat(dplys), (dply, callback) => {
                removeDply.one(docker, logger, dply, options, callback);
            }, callback);
        },
        (callback) => {
            if (names.length > 0) {
                callback(null);
                return;
            }
            let name = apemandocker.nameNetwork(apemanfile.$pkg);
            removeDply.network(docker, logger, name, options, callback);
        }
    ], callback);
}

removeDply.network = function (docker, logger, name, options, callback) {
    docker.networkExists(name, (exists) => {
        if (exists) {
            async.series([
                (callback) => {
                    docker.removeNetwork(name, {}, callback);
                },
                (callback) => {
                    logger.logNetworkRemoved(name);
                    callback(null);
                }
            ], callback);
        } else {
            callback(null);
        }
    });
};

removeDply.one = function (docker, logger, dply, options, callback) {
    let name = dply.name,
        container = dply.container;
    async.series([
        (callback) => {
            logger.logRemoveStarted(name);
            callback(null);
        },
        (callback) => {
            docker.containerExists(container, (exists) => {
                console.log('exists',exists);
                if (exists) {
                    docker.removeContainer(container, {
                        force: !!options.force
                    }, callback);
                } else {
                    callback(null);
                }
            });
        }
    ], (err) => {
        if (err) {

        } else {
            logger.logRemoveDone(name);
        }
        callback(err);
    });
};

module.exports = removeDply;
