/**
 * @function runDply
 */
"use strict";

const argx = require('argx'),
    path = require('path'),
    async = require('async'),
    apemandocker = require('apemandocker'),
    getDplys = require('./get_dplys'),
    RunLogger = require('../logging/run_logger');

/** @lends runDply */
function runDply(apemanfile, names, options, callback) {
    let args = argx(arguments);
    callback = args.pop('function') || argx.noop;
    options = args.pop('object') || {};

    let docker = apemandocker.create({}),
        logger = new RunLogger({}),
        dplys = getDplys(apemanfile, names);

    let networkName = apemandocker.nameNetwork(apemanfile.$pkg);
    async.series([
        (callback) => {
            runDply.network(docker, logger, networkName, {}, callback);
        },
        (callback) => {
            async.eachSeries([].concat(dplys), (dply, callback) => {
                runDply.one(docker, logger, dply, {
                    net: networkName
                }, callback);
            }, callback);
        }
    ], callback);
}

runDply.network = function (docker, logger, name, options, callback) {
    docker.networkExists(name, (exists) => {
        if (exists) {
            callback(null);
        } else {
            async.series([
                (callback) => {
                    docker.createNetwork(name, {
                    }, callback);
                },
                (callback) => {
                    logger.logNetworkCreated(name);
                    callback(null);
                }
            ], callback);
        }
    });
};

runDply.one = function (docker, logger, dply, options, callback) {
    let container = dply.container,
        name = dply.name,
        image = dply.image;

    let mount = dply.mount,
        port = dply.port,
        env = dply.env,
        hostname = dply.hostname;

    async.series([
        (callback) => {
            logger.logRunStarted(name, dply);
            callback(null);
        },
        (callback) => {
            docker.containerExists(container, (exists) => {
                if (exists) {
                    docker.isContainerRunning(container, (running)=> {
                        if (running) {
                            logger.warnAlreadyRunning(container);
                            callback(null);
                        } else {
                            docker.startContainer(container, {
                                attach: 'false'
                            }, callback);
                        }
                    });
                } else {
                    docker.runContainer(image, {
                        name: container,
                        detach: true,
                        hostname: hostname,
                        net: options.net,
                        publish: port && Object.keys(port).map((key) => {
                            return [port[key], key].join(':');
                        }),
                        volume: mount && Object.keys(mount).map((key) => {
                            return [path.resolve(mount[key]), key].join(':');
                        }),
                        env: env && Object.keys(env).map((key) => {
                            return [key, env[key]].join('=');
                        })
                    }, callback);
                }
            });
        }
    ], (err) => {
        if (err) {
        } else {
            logger.logRunDone(name);
        }
        callback(err);
    });
};

module.exports = runDply;
