/**
 * Logger for run.
 * @constructor RunLogger
 */

"use strict";

const colorprint = require('colorprint'),
    path = require('path'),
    arrayfilter = require('arrayfilter'),
    colors = colorprint.colors,
    textconv = require('textconv');


/** @lends RunLogger */
module.exports = colorprint.define(
    /** @lends RunLogger.prototype */
    {
        logNetworkCreated: function (name) {
            let s = this;
            s.info(`Network created: ${name}`);
        },
        logRunStarted: function (name, dply) {
            let s = this;
            s.info(`Deploying ${name}...`);

            let mount = dply.mount,
                port = dply.port,
                env = dply.env,
                infr = dply.infr,
                hostname = dply.hostname;

            if (infr) {
                s.debug('  Infr:');
                s.debug(`    ${infr}`);
            }

            if (mount) {
                s.debug('  Mount directory:');
                Object.keys(mount).forEach((key) => {
                    s.debug(`    ${mount[key]} => ${key}`);
                });
                s.debug('');
            }

            if (hostname) {
                s.debug('  Hostname:');
                s.debug(`    ${hostname}`);
                s.debug('');
            }

            if (port) {
                s.debug('  Publish port:');
                Object.keys(port).forEach((key) => {
                    s.debug(`    ${port[key]} => ${key}`);
                });
                s.debug('');
            }

            if (env) {
                s.debug('  Env variables:');
                Object.keys(env).forEach((key) => {
                    s.debug(`    ${key}: ${env[key]}`);
                });
                s.debug('');
            }

            s.debug('');
        },
        logRunDone: function (name) {
            let s = this;
            s.info(`...deploying ${name} done!`);
            s.debug('');
        },
        warnAlreadyRunning: function (name) {
            let s = this;
            s.warn(`Container already runing: ${name}`);
        }
    }
);
